@@ -9,6 +9,7 @@ Created on Aug 16, 2016
 import os
 import re
 import sys
+import yaml
 import logging
 from copy import deepcopy
 from avi.sdk.avi_api import ApiSession, ObjectNotFound, avi_sdk_syslog_logger, \
@@ -71,13 +72,13 @@ def ansible_return(module, rsp, changed, req=None, existing_obj=None,
     obj_val = rsp.json() if rsp else existing_obj
 
     if (obj_val and module.params.get("obj_username", None) and
-                "username" in obj_val):
+            "username" in obj_val):
         obj_val["obj_username"] = obj_val["username"]
     if (obj_val and module.params.get("obj_password", None) and
-                "password" in obj_val):
+            "password" in obj_val):
         obj_val["obj_password"] = obj_val["password"]
     if (obj_val and module.params.get("obj_state", None) and
-                "state" in obj_val):
+            "state" in obj_val):
         obj_val["obj_state"] = obj_val["state"]
     old_obj_val = existing_obj if changed and existing_obj else None
     api_context_val = api_context if disable_fact else None
@@ -147,18 +148,20 @@ def cleanup_absent_fields(obj):
         del obj[k]
     return obj
 
+
 def get_unicode_type():
     if sys.version_info < (3, 3):
         return unicode
     else:
         return str
 
-RE_REF_MATCH = re.compile('^/api/[\w/]+\?name\=[\w*]+[^#<>]*$')
+
+RE_REF_MATCH = re.compile(r'^/api/[\w/]+\?name\=[\w*]+[^#<>]*$')
 
 # if HTTP ref match then strip out the #name
 # HTTP_REF_MATCH = re.compile('https://[\w.0-9:-]+/api/[\w/\?.#&-]*$')
-HTTP_REF_MATCH = re.compile('https://[\w.0-9:-]+/api/.+')
-HTTP_REF_W_NAME_MATCH = re.compile('https://[\w.0-9:-]+/api/.*#.+')
+HTTP_REF_MATCH = re.compile(r'https://[\w.0-9:-]+/api/.+')
+HTTP_REF_W_NAME_MATCH = re.compile(r'https://[\w.0-9:-]+/api/.*#.+')
 
 
 def ref_n_str_cmp(x, y):
@@ -217,6 +220,7 @@ def ref_n_str_cmp(x, y):
                   x, y, y_name, y_uuid)
     return result
 
+
 def avi_obj_cmp(x, y, sensitive_fields=None):
     """
     compares whether x is fully contained in y. The comparision is different
@@ -299,7 +303,7 @@ def avi_obj_cmp(x, y, sensitive_fields=None):
                 elif not v:
                     d_x_absent_ks.append(k)
             elif isinstance(v, list) and not v and k not in y:
-                    d_x_absent_ks.append(k)
+                d_x_absent_ks.append(k)
             # Added condition to check key in dict.
             elif isinstance(v, str) or (k in y and isinstance(y[k], str)):
                 # this is the case when ansible converts the dictionary into a
@@ -328,8 +332,8 @@ def avi_obj_cmp(x, y, sensitive_fields=None):
 
 
 POP_FIELDS = ['state', 'controller', 'username', 'password', 'api_version',
-              'avi_credentials', 'avi_api_update_method', 'avi_api_patch_op',
-              'api_context', 'tenant', 'tenant_uuid', 'avi_disable_session_cache_as_fact']
+              'avi_credentials', 'avi_api_update_method', 'avi_api_patch_op', 'avi_patch_path',
+              'avi_patch_value', 'api_context', 'tenant', 'tenant_uuid', 'avi_disable_session_cache_as_fact']
 
 
 def get_api_context(module, api_creds):
@@ -348,6 +352,7 @@ def get_api_context(module, api_creds):
 NO_UUID_OBJ = ['cluster', 'systemconfiguration']
 SKIP_DELETE_ERROR = ["Cannot delete system default object", "Method \'DELETE\' not allowed"]
 
+
 def avi_ansible_api(module, obj_type, sensitive_fields):
     """
     This converts the Ansible module into AVI object and invokes APIs
@@ -388,7 +393,8 @@ def avi_ansible_api(module, obj_type, sensitive_fields):
     # Get the api version.
     avi_update_method = module.params.get('avi_api_update_method', 'put')
     avi_patch_op = module.params.get('avi_api_patch_op', 'add')
-
+    avi_patch_path = module.params.get('avi_patch_path')
+    avi_patch_value = module.params.get('avi_patch_value', None)
     api_version = api_creds.api_version
     name = module.params.get('name', None)
     # Added Support to get uuid
@@ -524,7 +530,19 @@ def avi_ansible_api(module, obj_type, sensitive_fields):
                 changed = True
             else:
                 obj.pop('name', None)
-                patch_data = {avi_patch_op: obj}
+                patch_data = {}
+                if avi_patch_path:
+                    if avi_patch_value:
+                        avi_patch_value = yaml.load(avi_patch_value)
+                    patch_data = {
+                        "json_patch": [{
+                            "op": avi_patch_op,
+                            "path": avi_patch_path,
+                            "value": avi_patch_value
+                        }]
+                    }
+                else:
+                    patch_data.update({avi_patch_op: obj})
                 try:
                     rsp = api.patch(
                         obj_path, data=patch_data, tenant=tenant,
@@ -559,7 +577,7 @@ def avi_common_argument_spec():
         controller=dict(default=os.environ.get('AVI_CONTROLLER', '')),
         username=dict(default=os.environ.get('AVI_USERNAME', '')),
         password=dict(default=os.environ.get('AVI_PASSWORD', ''), no_log=True),
-        api_version=dict(default='16.4.4', type='str'),
+        api_version=dict(default='18.2.6', type='str'),
         tenant=dict(default='admin'),
         tenant_uuid=dict(default='', type='str'),
         port=dict(type='int'),
@@ -575,10 +593,8 @@ def avi_common_argument_spec():
         password=dict(default=os.environ.get('AVI_PASSWORD', ''), no_log=True),
         tenant=dict(default='admin'),
         tenant_uuid=dict(default=''),
-        api_version=dict(default='16.4.4', type='str'),
+        api_version=dict(default='18.2.6', type='str'),
         avi_credentials=dict(default=None, type='dict',
                              options=credentials_spec),
         api_context=dict(type='dict'),
         avi_disable_session_cache_as_fact=dict(default=False, type='bool'))
-
-
